import javax.microedition.lcdui.*;
import java.util.*;

public class TeachForm extends Form implements CommandListener, ItemStateListener { 
    Dictionary  dictionary;
    ChoiceGroup variants;    
    boolean[]   flags;
    Random      rand;
    Vector      repeat;
    int         lang;
    int[]       nWords;
    int         nAttempts;
    int         counter;
    int         correctIndex;
    Pair        question;
    
    static final int N_VARIANTS = 4;
    /**
     * Interval of repeating one of the words for which user didn't choose correct translation
     * from the first attempt
     */
    static final int REPEAT_ITERVAL = 5;
    /**
     * Maximal number of not remembered words, knowledge of which should be confirmed.
     * When size of repeat list reachs this limit, no new word will be suggested
     */
    static final int REPEAT_THRESHOLD = 10;

    TeachForm(Dictionary dictionary, boolean[] flags) { 
        super(Locale.CHOOSE);
        this.dictionary = dictionary;
        this.flags = flags;
        repeat = new Vector();
        nWords = new int[2];
        variants = new ChoiceGroup(null, Choice.MULTIPLE);
        rand = new Random();
        for (int i = 0; i < N_VARIANTS; i++) { 
            variants.append("?", null);
        }
        for (int lang = 0; lang <= 1; lang++) { 
            nWords[lang] = dictionary.volumes[lang][dictionary.nVolumes[lang]-1].index;
        }
        append(variants);
        update();
        setItemStateListener(this);

        addCommand(Dictionary.TRANSLATE_CMD);
        addCommand(Dictionary.QUIT_CMD);
        setCommandListener(this);
        Display.getDisplay(dictionary).setCurrent(this);
    }

    int next(int mod) { 
        return (rand.nextInt() & Integer.MAX_VALUE) % mod;
    }

    private void update() { 
        do { 
            lang = 1-lang;
        } while (!flags[lang]);
       
        if (repeat.size() > REPEAT_THRESHOLD || 
            (repeat.size() != 0 && ++counter == REPEAT_ITERVAL)) 
        { 
            counter = 0;
            int pos = next(repeat.size());
            question = (Pair)repeat.elementAt(pos);
            repeat.removeElementAt(pos);
        } else { 
            question = dictionary.getWordPair(lang, next(nWords[lang]));
        }
        variants.setLabel(question.word);
        correctIndex = next(N_VARIANTS);
        for (int i = 0; i < N_VARIANTS; i++) { 
            int index = i == correctIndex ? question.index : next(nWords[1-lang]);
            variants.set(i, dictionary.getWordPair(1-lang, index).word, null);
        }
        variants.setSelectedFlags(new boolean[N_VARIANTS]);
        nAttempts = 0;
    }
       
    public void itemStateChanged(Item item) {
        boolean[] flags = new boolean[N_VARIANTS];
        variants.getSelectedFlags(flags);
        if (!flags[correctIndex]) { 
            nAttempts += 1;
            Alert a = new Alert(Locale.ERROR, Locale.NOT_CORRECT, null, AlertType.ERROR);
            Display.getDisplay(dictionary).setCurrent(a, this);
        } else { 
            if (nAttempts != 0) { 
                repeat.addElement(question);
            }
            update();
        }
    }

    public void commandAction(Command c, Displayable d) 
    {
        if (c == Dictionary.QUIT_CMD) {
            dictionary.destroyApp(true);
            dictionary.notifyDestroyed();
        } else if (c == Dictionary.TRANSLATE_CMD) { 
            new TranslateForm(dictionary, this, question.word);
        }
    }
}